/// <reference types="node" />
import { EventEmitter } from "events";
interface SchedulerOptions<T> {
    threads: number;
    autoStop?: boolean;
    taskErrorHandler?: (err: Error, task: SchedulerTask<T>) => TaskFailDecision;
}
interface Task<T> {
    handler: (payload: T) => Promise<any>;
    payload?: T;
}
export interface SchedulerTask<T = Record<string, unknown>> extends Task<T> {
    priority: number;
    retryCount: number;
    uuid: string;
}
export interface TaskFinishEvent<T> {
    task: SchedulerTask<T>;
    finishCount: number;
    totalCount: number;
    dropCount: number;
}
export interface TaskErrorEvent<T, E = Error> {
    task: SchedulerTask<T>;
    error: E;
    decision: TaskFailDecision;
}
export interface TaskDropEvent<T, E = Error> {
    task: SchedulerTask<T>;
    error: E;
}
/** 任务失败决策 */
export declare enum TaskFailDecision {
    /** 重试 */
    RETRY = 0,
    /** 放弃 */
    DROP = 1,
    /** 提升优先级并重试 */
    INCREASE_PRIORITY = 2
}
declare class TaskScheduler<T> extends EventEmitter {
    private queue;
    private nowRunningThreadsCount;
    private threads;
    private isFinished;
    private finishCount;
    private dropCount;
    private totalCount;
    private taskErrorHandler;
    constructor(options: SchedulerOptions<T>);
    get isQueueEmpty(): boolean;
    private checkQueue;
    private getNextTask;
    private runTask;
    addTasks(tasks: Task<T> | Task<T>[]): void;
    start(): void;
    waitAllFinished(): Promise<void>;
}
export default TaskScheduler;
