"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
var PriorityQueue = /** @class */ (function () {
    function PriorityQueue() {
        this.heap = [];
    }
    Object.defineProperty(PriorityQueue.prototype, "size", {
        get: function () {
            return this.heap.length;
        },
        enumerable: false,
        configurable: true
    });
    PriorityQueue.prototype.getParentIndex = function (index) {
        return Math.floor(index / 2);
    };
    PriorityQueue.prototype.getLeftChildIndex = function (index) {
        return index * 2;
    };
    PriorityQueue.prototype.getRightChildIndex = function (index) {
        return index * 2 + 1;
    };
    PriorityQueue.prototype.swap = function (index1, index2) {
        var temp = this.heap[index1];
        this.heap[index1] = this.heap[index2];
        this.heap[index2] = temp;
    };
    PriorityQueue.prototype.shiftUp = function (index) {
        while (index > 0 && this.heap[this.getParentIndex(index)].priority < this.heap[index].priority) {
            this.swap(index, this.getParentIndex(index));
            this.shiftUp(this.getParentIndex(index));
        }
    };
    PriorityQueue.prototype.shiftDown = function (index) {
        var targetIndex = index;
        var leftChildIndex = this.getLeftChildIndex(index);
        var rightChildIndex = this.getRightChildIndex(index);
        if (leftChildIndex <= this.size - 1 && this.heap[leftChildIndex].priority > this.heap[targetIndex].priority) {
            targetIndex = leftChildIndex;
        }
        if (rightChildIndex <= this.size - 1 && this.heap[rightChildIndex].priority > this.heap[targetIndex].priority) {
            targetIndex = rightChildIndex;
        }
        if (targetIndex !== index) {
            this.swap(targetIndex, index);
            this.shiftDown(targetIndex);
        }
    };
    PriorityQueue.prototype.insert = function (item) {
        this.heap.push(item);
        this.shiftUp(this.size - 1);
    };
    PriorityQueue.prototype.insertMulti = function (items) {
        var e_1, _a;
        try {
            for (var items_1 = __values(items), items_1_1 = items_1.next(); !items_1_1.done; items_1_1 = items_1.next()) {
                var item = items_1_1.value;
                this.insert(item);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (items_1_1 && !items_1_1.done && (_a = items_1.return)) _a.call(items_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    PriorityQueue.prototype.getMaxPriorityItem = function () {
        var result = this.heap[0];
        this.heap[0] = this.heap[this.size - 1];
        this.heap.pop();
        this.shiftDown(0);
        return result;
    };
    return PriorityQueue;
}());
exports.default = PriorityQueue;
//# sourceMappingURL=priority_queue.js.map